@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # coding: utf-8
 
-# In[94]:
+# In[2]:
 
 
 try:
@@ -12,10 +12,10 @@ except:
         pass
 
 
-# In[95]:
+# In[3]:
 
 
-version = '0.53'
+version = '0.54'
 
 import re
 import tempfile
@@ -28,11 +28,12 @@ import shutil
 import time
 import os
 import stat
+import pickle
 
 
 # # Helper Functions
 
-# In[96]:
+# In[4]:
 
 
 def is_testing():
@@ -42,7 +43,7 @@ def is_testing():
         return False
 
 
-# In[97]:
+# In[5]:
 
 
 def isnotebook():
@@ -53,7 +54,7 @@ def isnotebook():
     return False
 
 
-# In[98]:
+# In[6]:
 
 
 def testPrint(*args):
@@ -63,7 +64,7 @@ def testPrint(*args):
 
 # # Requiremtns
 
-# In[99]:
+# In[7]:
 
 
 requirements = []
@@ -90,16 +91,48 @@ requirements.append('ipython')
 from IPython.core.display import display, HTML
 
 
+# # Strings
+
+# In[8]:
+
+
+def sFloat0(f): return "{:0.0f}".format(f)
+def sFloat1(f): return "{:0.1f}".format(f)
+def sFloat2(f): return "{:0.2f}".format(f)
+def sFloat3(f): return "{:0.3f}".format(f)
+def sFloat4(f): return "{:0.4f}".format(f)
+def sFloat5(f): return "{:0.5f}".format(f)
+def sFloatSci0(f): return "{:0.0E}".format(f)
+def sFloatSci1(f): return "{:0.1E}".format(f)
+def sFloatSci2(f): return "{:0.2E}".format(f)
+def sFloatSci3(f): return "{:0.3E}".format(f)
+def sFloatSci4(f): return "{:0.4E}".format(f)
+def sFloatSci5(f): return "{:0.5E}".format(f)
+
+if is_testing():
+    print(sFloat0(1.234567))
+    print(sFloat1(1.234567))
+    print(sFloat2(1.234567))
+    print(sFloat3(1.234567))
+    print(sFloat4(1.234567))
+    print(sFloatSci0(1.234567))
+    print(sFloatSci1(1.234567))
+    print(sFloatSci2(1.234567))
+    print(sFloatSci3(1.234567))
+    print(sFloatSci4(1.234567))
+    print(sFloatSci5(1.234567))
+
+
 # # Lists
 
-# In[100]:
+# In[9]:
 
 
 def listsGetPairsFromTwoLists(list1, list2):
     return list(zip(list1, list2))
 
 
-# In[101]:
+# In[10]:
 
 
 if is_testing():
@@ -109,14 +142,14 @@ if is_testing():
     print(listsGetPairsFromTwoLists(listAbcd, list1234))
 
 
-# In[102]:
+# In[11]:
 
 
 def listSequentialOverlappingPairs(l):
     return list(zip(l[:-1], l[1:]))
 
 
-# In[103]:
+# In[12]:
 
 
 if is_testing():
@@ -132,7 +165,7 @@ if is_testing():
 
 # # List of Strings
 
-# In[104]:
+# In[13]:
 
 
 regex_cache = {}
@@ -142,35 +175,35 @@ def sListFilter(l, re_include=".*", re_exclude='^\b$'):
     return [e for e in l if regex_cache[re_include].search(e) and not regex_cache[re_exclude].search(e)]
 
 
-# In[105]:
+# In[14]:
 
 
 if is_testing():
     sListFilter(['good x', 'other good x', 'without the letter before y'])
 
 
-# In[106]:
+# In[15]:
 
 
 if is_testing():
     print(sListFilter(['good x', 'other good x', 'without the letter before y'], re_include='x'))
 
 
-# In[107]:
+# In[16]:
 
 
 if is_testing():
     print(sListFilter(['not enough x', 'enough xx', 'enough xx and also z'], re_include='xx+'))
 
 
-# In[108]:
+# In[17]:
 
 
 if is_testing():
     print(sListFilter(['not enough x', 'enough xx', 'enough xx and also z'], re_include='xx+', re_exclude='z'))
 
 
-# In[109]:
+# In[18]:
 
 
 def lCsv(l, force_oneline=False):
@@ -186,21 +219,21 @@ def lCsv(l, force_oneline=False):
     return ", ".join([str(e) for e in l])
 
 
-# In[110]:
+# In[19]:
 
 
 if is_testing():
     print(lCsv(['a', 'b', 1, 3.14]))
 
 
-# In[111]:
+# In[20]:
 
 
 if is_testing():
     print(lCsv([['a', 'b'], 1, 3.14]))
 
 
-# In[112]:
+# In[21]:
 
 
 if is_testing():
@@ -209,7 +242,7 @@ if is_testing():
 
 # # Text files
 
-# In[113]:
+# In[22]:
 
 
 def txtread(path):
@@ -230,7 +263,7 @@ def txtwrite(path, txt):
             return f.write(txt)
 
 
-# In[114]:
+# In[23]:
 
 
 if is_testing():
@@ -239,7 +272,7 @@ if is_testing():
 
 # # List of Strings
 
-# In[115]:
+# In[24]:
 
 
 regex_cache = {}
@@ -251,35 +284,35 @@ def sListFilter(listOfStrings, re_include=".*", re_exclude='^\b$'):
 
 # ### Examples
 
-# In[116]:
+# In[25]:
 
 
 if is_testing():
     print(sListFilter(['good x', 'other good x', 'without the letter before y']))
 
 
-# In[117]:
+# In[26]:
 
 
 if is_testing():
     print(sListFilter(['good x', 'other good x', 'without the letter before y'], re_include='x'))
 
 
-# In[118]:
+# In[27]:
 
 
 if is_testing():
     print(sListFilter(['not enough x', 'enough xx', 'enough xx and also z'], re_include='xx+'))
 
 
-# In[119]:
+# In[28]:
 
 
 if is_testing():
     print(sListFilter(['not enough x', 'enough xx', 'enough xx and also z'], re_include='xx+', re_exclude='z'))
 
 
-# In[120]:
+# In[29]:
 
 
 if is_testing():
@@ -288,7 +321,7 @@ if is_testing():
 
 # # GCP Google Cloud Platform
 
-# In[121]:
+# In[30]:
 
 
 import warnings
@@ -312,7 +345,7 @@ def _gsBlob(path):
     return blob
 
 
-# In[122]:
+# In[31]:
 
 
 def gsGsToHttp(path):
@@ -322,7 +355,7 @@ def gsHttpToGs(path):
     return path.replace("https://storage.cloud.google.com/", "gs://").split("?")[0] # "?" is for possible "?authuser=0"
 
 
-# In[123]:
+# In[32]:
 
 
 def gsLs(path):
@@ -332,7 +365,7 @@ def gsLs(path):
     return ['gs://' + bucket + '/' + blob.name for blob in client.list_blobs(prefix=prefix)]
 
 
-# In[124]:
+# In[33]:
 
 
 def gsUploadFile(gsPath, path):
@@ -340,7 +373,7 @@ def gsUploadFile(gsPath, path):
     blob.upload_from_filename(path)
 
 
-# In[125]:
+# In[34]:
 
 
 def gsTxtread(path):
@@ -352,7 +385,7 @@ def gsTxtread(path):
     return blob.download_as_string().decode()
 
 
-# In[126]:
+# In[35]:
 
 
 def gsTxtwrite(path, s):
@@ -360,7 +393,7 @@ def gsTxtwrite(path, s):
     return blob.upload_from_string(s)
 
 
-# In[127]:
+# In[36]:
 
 
 def gsImwrite(path, img):
@@ -369,7 +402,7 @@ def gsImwrite(path, img):
     blob.upload_from_string(encimg.tobytes())
 
 
-# In[128]:
+# In[37]:
 
 
 def gsImread(path):
@@ -378,9 +411,77 @@ def gsImread(path):
     return cv2.imdecode(raw, 1)[:, :, [2, 1, 0]]
 
 
+# In[38]:
+
+
+def gsPklWrite(path, obj):
+    blob = _gsBlob(path)
+    return blob.upload_from_string(pickle.dumps(obj, protocol=4))
+
+
+# In[39]:
+
+
+def gsPklRead(path):
+    blob = _gsBlob(path)
+    return pickle.loads(blob.download_as_string())
+
+
+# # Pickle
+
+# In[40]:
+
+
+def pklwrite(path, obj):
+    if 'gs://' in path:
+        return gsPklWrite(path, obj)
+    else:
+        with open(path, 'wb') as f:
+            return pickle.dump(obj, f, protocol=4)
+
+
+# In[41]:
+
+
+def pklread(path):
+    if 'gs://' in path:
+        return gsPklRead(path)
+    else:
+        with open(path, 'rb') as f:
+            return pickle.load(f)
+
+
+# In[42]:
+
+
+if is_testing():
+    with tempfile.TemporaryDirectory() as dir_path:
+        pkl_path = os.path.join(dir_path, 'test.pkl')
+        
+        pklwrite(pkl_path, 'test')
+        print(pklread(pkl_path))
+
+
+# In[ ]:
+
+
+for p in [
+    #"gs://srfl-data/DIV2K-te_X8.pklv4",
+    #"gs://srfl-data/DIV2K-te_X4.pklv4",
+    #"gs://srfl-data/DIV2K-te.pklv4",
+    "gs://srfl-data/CelebAHq_160_MBic_te_X16.pklv4",
+    "gs://srfl-data/CelebAHq_160_MBic_te_X8.pklv4",
+    "gs://srfl-data/CelebAHq_160_MBic_te_X4.pklv4",
+    "gs://srfl-data/CelebAHq_160_MBic_te.pklv4"
+    ]:
+    l = pklread(p)
+    l = l[:int(np.ceil(len(l)*0.1))]
+    pklwrite(p.replace(".pklv4", "_10pct.pklv4"), l)
+
+
 # # Images
 
-# In[129]:
+# In[ ]:
 
 
 def imread(path):
@@ -414,7 +515,7 @@ def impad(img, top=0, bottom=0, left=0, right=0, color=255):
     return np.pad(img, [(top, bottom), (left, right), (0, 0)], 'constant', constant_values=color)
 
 
-# In[130]:
+# In[ ]:
 
 
 def imshow(array, scale=1):
@@ -429,7 +530,7 @@ def imshow(array, scale=1):
         cv2.waitKey()
 
 
-# In[131]:
+# In[ ]:
 
 
 if is_testing():
